@@ -2,7 +2,7 @@
  * commit_util.c:  Driver for the WC commit process.
  *
  * ====================================================================
- * Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -34,7 +34,7 @@
 #include <assert.h>
 #include <stdlib.h>  /* for qsort() */
 
-
+#include "svn_private_config.h"
 
 /*** Uncomment this to turn on commit driver debugging. ***/
 /*
@@ -189,6 +189,8 @@ harvest_committables (apr_hash_t *committables,
   svn_boolean_t tc, pc;
   const char *cf_url = NULL;
   svn_revnum_t cf_rev = entry->copyfrom_rev;
+  const svn_string_t *propval;
+  svn_boolean_t is_special;
 
   /* Early out if the item is already marked as committable. */
   if (look_up_committable (committables, path, pool))
@@ -208,16 +210,31 @@ harvest_committables (apr_hash_t *committables,
      entry was written. */
   if ((entry->kind != svn_node_file) && (entry->kind != svn_node_dir))
     return svn_error_createf
-      (SVN_ERR_NODE_UNKNOWN_KIND, NULL, "Unknown entry kind for \"%s\"", path);
+      (SVN_ERR_NODE_UNKNOWN_KIND, NULL, _("Unknown entry kind for '%s'"), path);
 
-  SVN_ERR (svn_io_check_path (path, &kind, pool));
+  SVN_ERR (svn_io_check_special_path (path, &kind, &is_special, pool));
 
   if ((kind != svn_node_file)
       && (kind != svn_node_dir)
       && (kind != svn_node_none))
     {
       return svn_error_createf
-        (SVN_ERR_NODE_UNKNOWN_KIND, NULL, "Unknown kind for \"%s\"", path);
+        (SVN_ERR_NODE_UNKNOWN_KIND, NULL,
+         _("Unknown entry kind for '%s'"), path);
+    }
+
+  /* Verify that the node's type has not changed before attempting to
+     commit. */
+  SVN_ERR (svn_wc_prop_get (&propval, SVN_PROP_SPECIAL, path, adm_access,
+                            pool));
+
+  if ((((! propval) && (is_special)) ||
+          ((propval) && (! is_special))) &&
+      (kind != svn_node_none))
+    {
+      return svn_error_createf
+        (SVN_ERR_NODE_UNEXPECTED_KIND, NULL,
+         _("Entry '%s' has unexpectedly changed special status"), path);
     }
 
   /* Get a fully populated entry for PATH if we can, and check for
@@ -267,7 +284,7 @@ harvest_committables (apr_hash_t *committables,
   /* Bail now if any conflicts exist for the ENTRY. */
   if (tc || pc)
     return svn_error_createf (SVN_ERR_WC_FOUND_CONFLICT, NULL,
-                              "Aborting commit: '%s' remains in conflict.",
+                              _("Aborting commit: '%s' remains in conflict"),
                               path);
 
   /* If we have our own URL, and we're NOT in COPY_MODE, it wins over
@@ -323,7 +340,7 @@ harvest_committables (apr_hash_t *committables,
       else if (! copy_mode)
         return svn_error_createf 
           (SVN_ERR_WC_CORRUPT, NULL,
-           "Did not expect `%s' to be a working copy root", path);
+           _("Did not expect '%s' to be a working copy root"), path);
 
       /* If the ENTRY's revision differs from that of its parent, we
          have to explicitly commit ENTRY as a copy. */
@@ -331,17 +348,16 @@ harvest_committables (apr_hash_t *committables,
         {
           state_flags |= SVN_CLIENT_COMMIT_ITEM_ADD;
           state_flags |= SVN_CLIENT_COMMIT_ITEM_IS_COPY;
-          adds_only = TRUE;
+          adds_only = FALSE;
           cf_rev = entry->revision;
           if (copy_mode)
             cf_url = entry->url;
           else if (copyfrom_url)
             cf_url = copyfrom_url;
-          else
+          else /* ### See issue #830 */
             return svn_error_createf 
               (SVN_ERR_BAD_URL, NULL,
-               "Commit item '%s' has copy flag but no copyfrom url\n"
-               "See issue #830", path);
+               _("Commit item '%s' has copy flag but no copyfrom URL\n"), path);
         }
     }
 
@@ -475,9 +491,11 @@ harvest_committables (apr_hash_t *committables,
                       /* A missing, schedule-delete child dir is
                          allowable.  Just don't try to recurse. */
                       svn_node_kind_t childkind;
-                      SVN_ERR (svn_io_check_path (full_path,
-                                                  &childkind, loop_pool));
-                      if (childkind == svn_node_none
+                      svn_error_t *err = svn_io_check_path (full_path,
+                                                            &childkind,
+                                                            loop_pool);
+                      if (! err
+                          && childkind == svn_node_none
                           && this_entry->schedule == svn_wc_schedule_delete)
                         {
                           add_committable (committables, full_path,
@@ -485,11 +503,15 @@ harvest_committables (apr_hash_t *committables,
                                            SVN_INVALID_REVNUM, 
                                            NULL,
                                            SVN_CLIENT_COMMIT_ITEM_DELETE);
+                          svn_error_clear (lockerr);
                           svn_pool_clear (loop_pool);
                           continue; /* don't recurse! */
                         }
                       else
-                        return lockerr;
+                        {
+                          svn_error_clear (err);
+                          return lockerr;
+                        }
                     }
                   else
                     return lockerr;
@@ -580,10 +602,12 @@ svn_client__harvest_committables (apr_hash_t **committables,
                                           target, subpool));
       SVN_ERR (svn_wc_entry (&entry, target, adm_access, FALSE, subpool));
       if (! entry)
-        return svn_error_create (SVN_ERR_ENTRY_NOT_FOUND, NULL, target);
+        return svn_error_createf
+          (SVN_ERR_ENTRY_NOT_FOUND, NULL,
+           _("'%s' is not under version control"), target);
       if (! entry->url)
         return svn_error_createf (SVN_ERR_WC_CORRUPT, NULL, 
-                                  "Entry for `%s' has no URL", target);
+                                  _("Entry for '%s' has no URL"), target);
 
       /* We have to be especially careful around entries scheduled for
          addition or replacement. */
@@ -601,8 +625,8 @@ svn_client__harvest_committables (apr_hash_t **committables,
           if (err && err->apr_err == SVN_ERR_WC_NOT_LOCKED)
             {
               svn_error_clear (err);
-              SVN_ERR (svn_wc_adm_open (&parent_access, NULL, parent,
-                                        FALSE, FALSE, subpool));
+              SVN_ERR (svn_wc_adm_open2 (&parent_access, NULL, parent,
+                                         FALSE, 0, subpool));
             }
           else if (err)
             {
@@ -614,8 +638,7 @@ svn_client__harvest_committables (apr_hash_t **committables,
           if (! p_entry)
             return svn_error_createf 
               (SVN_ERR_WC_CORRUPT, NULL, 
-               "Entry for `%s' is scheduled for addition, yet its parent\n"
-               "directory does not appear to be under version control", 
+               _("'%s' is scheduled for addition within unversioned parent"),
                target);
           if ((p_entry->schedule == svn_wc_schedule_add)
               || (p_entry->schedule == svn_wc_schedule_replace))
@@ -635,9 +658,9 @@ svn_client__harvest_committables (apr_hash_t **committables,
       if ((entry->copied) && (entry->schedule == svn_wc_schedule_normal))
         return svn_error_createf 
           (SVN_ERR_ILLEGAL_TARGET, NULL, 
-           "Entry for `%s' is marked as `copied' but is not itself scheduled\n"
-           "for addition.  Perhaps you're committing a target that this\n"
-           "inside of an unversioned (or not-yet-versioned) directory?",
+           _("Entry for '%s' is marked as 'copied' but is not itself scheduled"
+             "\nfor addition.  Perhaps you're committing a target that is\n"
+             "inside an unversioned (or not-yet-versioned) directory?"),
            target);
 
       /* Handle our TARGET. */
@@ -675,9 +698,9 @@ svn_client__harvest_committables (apr_hash_t **committables,
           {
             return svn_error_createf 
               (SVN_ERR_ILLEGAL_TARGET, NULL, 
-               "`%s' is not versioned in the repository "
-               "and is not part of the commit, "
-               "yet its child '%s' is part of the commit.",
+               _("'%s' is not under version control "
+                 "and is not part of the commit, "
+                 "yet its child '%s' is part of the commit"),
                dangling_parent, dangling_child);
           }
       }
@@ -705,8 +728,9 @@ svn_client__get_copy_committables (apr_hash_t **committables,
   /* Read the entry for TARGET. */
   SVN_ERR (svn_wc_entry (&entry, target, adm_access, FALSE, pool));
   if (! entry)
-    return svn_error_create 
-      (SVN_ERR_ENTRY_NOT_FOUND, NULL, target);
+    return svn_error_createf
+      (SVN_ERR_ENTRY_NOT_FOUND, NULL, _("'%s' is not under version control"),
+       target);
       
   /* Handle our TARGET. */
   SVN_ERR (harvest_committables (*committables, target, adm_access,
@@ -754,7 +778,7 @@ svn_client__condense_commit_items (const char **base_url,
       if ((last_item) && (strcmp (last_item->url, url) == 0))
         return svn_error_createf 
           (SVN_ERR_CLIENT_DUPLICATE_COMMIT_URL, NULL,
-           "Cannot commit both `%s' and `%s' as they refer to the same URL.",
+           _("Cannot commit both '%s' and '%s' as they refer to the same URL"),
            item->path, last_item->path);
 
       /* In the first iteration, our BASE_URL is just our only
@@ -800,7 +824,7 @@ svn_client__condense_commit_items (const char **base_url,
 
 #ifdef SVN_CLIENT_COMMIT_DEBUG
   /* ### TEMPORARY CODE ### */
-  fprintf (stderr, "COMMITTABLES: (base url=%s)\n", *base_url);
+  fprintf (stderr, "COMMITTABLES: (base URL=%s)\n", *base_url);
   fprintf (stderr, "   FLAGS     REV  REL-URL (COPY-URL)\n");
   for (i = 0; i < ci->nelts; i++)
     {
@@ -818,7 +842,7 @@ svn_client__condense_commit_items (const char **base_url,
       flags[4] = (this_item->state_flags & SVN_CLIENT_COMMIT_ITEM_IS_COPY)
                    ? 'c' : '-';
       flags[5] = '\0';
-      fprintf (stderr, "   %s  %6" SVN_REVNUM_T_FMT "  `%s' (%s)\n", 
+      fprintf (stderr, "   %s  %6ld  '%s' (%s)\n", 
                flags,
                this_item->revision,
                this_item->url ? this_item->url : "",
@@ -832,32 +856,34 @@ svn_client__condense_commit_items (const char **base_url,
 
 struct file_mod_t
 {
-  apr_pool_t *subpool;
   svn_client_commit_item_t *item;
   void *file_baton;
 };
 
 
+/* A baton for use with the path-based editor driver */
 struct path_driver_cb_baton
 {
-  svn_wc_adm_access_t *adm_access;
-  const svn_delta_editor_t *editor;
-  void *edit_baton;
-  apr_hash_t *file_mods;
-  apr_hash_t *tempfiles;
-  const char *notify_path_prefix;
-  svn_client_ctx_t *ctx;
-  apr_hash_t *commit_items;
+  svn_wc_adm_access_t *adm_access;     /* top-level access baton */
+  const svn_delta_editor_t *editor;    /* commit editor */
+  void *edit_baton;                    /* commit editor's baton */
+  apr_hash_t *file_mods;               /* hash: path->file_mod_t */
+  apr_hash_t *tempfiles;               /* hash of tempfiles created */
+  const char *notify_path_prefix;      /* notification path prefix */
+  svn_client_ctx_t *ctx;               /* client context baton */
+  apr_hash_t *commit_items;            /* the committables */
 };
 
 
+/* This implements svn_delta_path_driver_cb_func_t */
 static svn_error_t *
 do_item_commit (void **dir_baton,
-                const char *path,
                 void *parent_baton,
-                struct path_driver_cb_baton *cb_baton,
+                void *callback_baton,
+                const char *path,
                 apr_pool_t *pool)
 {
+  struct path_driver_cb_baton *cb_baton = callback_baton;
   svn_client_commit_item_t *item = apr_hash_get (cb_baton->commit_items,
                                                  path, APR_HASH_KEY_STRING);
   svn_node_kind_t kind = item->kind;
@@ -875,8 +901,16 @@ do_item_commit (void **dir_baton,
   *dir_baton = NULL;
   if (item->copyfrom_url)
     copyfrom_url = item->copyfrom_url;
-  if (kind == svn_node_file)
-    file_pool = svn_pool_create (apr_hash_pool_get (file_mods));
+
+  /* If this is a file with textual mods, we'll be keeping its baton
+     around until the end of the commit.  So just lump its memory into
+     a single, big, all-the-file-batons-in-here pool.  Otherwise, we
+     can just use POOL, and trust our caller to clean that mess up. */
+  if ((kind == svn_node_file) 
+      && (item->state_flags & SVN_CLIENT_COMMIT_ITEM_TEXT_MODS))
+    file_pool = apr_hash_pool_get (file_mods);
+  else
+    file_pool = pool;
 
   /* Call the cancellation function. */
   if (ctx->cancel_func)
@@ -888,11 +922,11 @@ do_item_commit (void **dir_baton,
       if (! copyfrom_url)
         return svn_error_createf 
           (SVN_ERR_BAD_URL, NULL,
-           "Commit item '%s' has copy flag but no copyfrom url", path);
+           _("Commit item '%s' has copy flag but no copyfrom URL"), path);
       if (! SVN_IS_VALID_REVNUM (item->revision))
         return svn_error_createf 
           (SVN_ERR_CLIENT_BAD_REVISION, NULL,
-           "Commit item '%s' has copy flag but an invalid revision", path);
+           _("Commit item '%s' has copy flag but an invalid revision"), path);
     }
 
   /* If a feedback table was supplied by the application layer,
@@ -1056,8 +1090,7 @@ do_item_commit (void **dir_baton,
   if ((kind == svn_node_file) 
       && (item->state_flags & SVN_CLIENT_COMMIT_ITEM_TEXT_MODS))
     {
-      struct file_mod_t *mod = apr_palloc (apr_hash_pool_get (file_mods),
-                                           sizeof (*mod));
+      struct file_mod_t *mod = apr_palloc (file_pool, sizeof (*mod));
 
       if (! file_baton)
         {
@@ -1068,36 +1101,21 @@ do_item_commit (void **dir_baton,
         }
 
       /* Add this file mod to the FILE_MODS hash. */
-      mod->subpool = file_pool;
       mod->item = item;
       mod->file_baton = file_baton;
       apr_hash_set (file_mods, item->url, APR_HASH_KEY_STRING, (void *)mod);
     }
-
-  /* Close any outstanding file batons that didn't get caught by the
-     "has local mods" conditional above. */
   else if (file_baton)
     {
+      /* Close any outstanding file batons that didn't get caught by
+         the "has local mods" conditional above. */
       SVN_ERR (editor->close_file (file_baton, NULL, file_pool));
-      svn_pool_destroy (file_pool);
     }
 
   return SVN_NO_ERROR;
 }
 
 
-static svn_error_t *
-path_driver_cb_func (void **dir_baton,
-                     void *parent_baton,
-                     void *callback_baton,
-                     const char *path,
-                     apr_pool_t *pool)
-{
-  struct path_driver_cb_baton *cb_baton = callback_baton;
-  return do_item_commit (dir_baton, path, parent_baton, cb_baton, pool);
-}
-
-
 #ifdef SVN_CLIENT_COMMIT_DEBUG
 /* Prototype for function below */
 static svn_error_t *get_test_editor (const svn_delta_editor_t **editor,
@@ -1121,6 +1139,7 @@ svn_client__do_commit (const char *base_url,
 {
   apr_hash_t *file_mods = apr_hash_make (pool);
   apr_hash_t *items_hash = apr_hash_make (pool);
+  apr_pool_t *subpool = svn_pool_create (pool);
   apr_hash_index_t *hi;
   int i;
   struct path_driver_cb_baton cb_baton;
@@ -1164,8 +1183,7 @@ svn_client__do_commit (const char *base_url,
 
   /* Drive the commit editor! */
   SVN_ERR (svn_delta_path_driver (editor, edit_baton, SVN_INVALID_REVNUM,
-                                  paths, path_driver_cb_func, 
-                                  &cb_baton, pool));
+                                  paths, do_item_commit, &cb_baton, pool));
 
   /* Transmit outstanding text deltas. */
   for (hi = apr_hash_first (pool, file_mods); hi; hi = apr_hash_next (hi))
@@ -1203,44 +1221,61 @@ svn_client__do_commit (const char *base_url,
       if (item->state_flags & SVN_CLIENT_COMMIT_ITEM_ADD)
         fulltext = TRUE;
 
-      dir_path = svn_path_dirname (item->path, mod->subpool);
+      dir_path = svn_path_dirname (item->path, subpool);
       SVN_ERR (svn_wc_adm_retrieve (&item_access, adm_access, dir_path,
-                                    mod->subpool));
+                                    subpool));
       SVN_ERR (svn_wc_transmit_text_deltas (item->path, item_access, fulltext,
                                             editor, file_baton, 
-                                            &tempfile, mod->subpool));
+                                            &tempfile, subpool));
       if (tempfile && *tempfiles)
         {
           tempfile = apr_pstrdup (apr_hash_pool_get (*tempfiles), tempfile);
           apr_hash_set (*tempfiles, tempfile, APR_HASH_KEY_STRING, (void *)1);
         }
-      
-      svn_pool_destroy (mod->subpool);
+
+      svn_pool_clear (subpool);
     }
 
+  svn_pool_destroy (subpool);
+
   /* Close the edit. */
   SVN_ERR (editor->close_edit (edit_baton, pool));
   return SVN_NO_ERROR;
 }
 
+/* Commit callback baton */
 
-svn_client_commit_info_t *
-svn_client__make_commit_info (svn_revnum_t revision,
-                              const char *author,
-                              const char *date,
-                              apr_pool_t *pool)
+struct commit_baton {
+  svn_client_commit_info_t **info;
+  apr_pool_t *pool;
+};
+
+svn_error_t *svn_client__commit_get_baton (void **baton,
+                                           svn_client_commit_info_t **info,
+                                           apr_pool_t *pool)
 {
-  svn_client_commit_info_t *info;
+  struct commit_baton *cb = apr_pcalloc (pool, sizeof (*cb));
+  cb->info = info;
+  cb->pool = pool;
+  *baton = cb;
 
-  if (date || author || SVN_IS_VALID_REVNUM (revision))
-    {
-      info = apr_palloc (pool, sizeof (*info));
-      info->date = date ? apr_pstrdup (pool, date) : NULL;
-      info->author = author ? apr_pstrdup (pool, author) : NULL;
-      info->revision = revision;
-      return info;
-    }
-  return NULL;
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *svn_client__commit_callback (svn_revnum_t revision,
+                                          const char *date,
+                                          const char *author,
+                                          void *baton)
+{
+  struct commit_baton *cb = baton;
+  svn_client_commit_info_t **info = cb->info;
+
+  *info = apr_palloc (cb->pool, sizeof (**info));
+  (*info)->date = date ? apr_pstrdup (cb->pool, date) : NULL;
+  (*info)->author = author ? apr_pstrdup (cb->pool, author) : NULL;
+  (*info)->revision = revision;
+
+  return SVN_NO_ERROR;
 }
 
 
@@ -1296,7 +1331,7 @@ open_root (void *edit_baton,
 {
   struct edit_baton *eb = edit_baton;
   struct item_baton *new_baton = make_baton (eb, NULL, eb->path, dir_pool);
-  fprintf (stderr, "TEST EDIT STARTED (base url=%s)\n", eb->path);
+  fprintf (stderr, "TEST EDIT STARTED (base URL=%s)\n", eb->path);
   *root_baton = new_baton;
   return (*eb->real_editor->open_root) (eb->real_eb,
                                         base_revision,
@@ -1317,7 +1352,7 @@ add_file (const char *path,
   const char *copystuffs = "";
   if (copyfrom_path && SVN_IS_VALID_REVNUM(copyfrom_revision))
     copystuffs = apr_psprintf (pool, 
-                               " (copied from %s:%" SVN_REVNUM_T_FMT ")",
+                               " (copied from %s:%ld)",
                                copyfrom_path,
                                copyfrom_revision);
   fprintf (stderr, "   Adding  : %s%s\n", path, copystuffs);
@@ -1412,7 +1447,7 @@ add_directory (const char *path,
   const char *copystuffs = "";
   if (copyfrom_path && SVN_IS_VALID_REVNUM(copyfrom_revision))
     copystuffs = apr_psprintf (pool, 
-                               " (copied from %s:%" SVN_REVNUM_T_FMT ")",
+                               " (copied from %s:%ld)",
                                copyfrom_path,
                                copyfrom_revision);
   fprintf (stderr, "   Adding  : %s%s\n", path, copystuffs);
@@ -1484,6 +1519,8 @@ get_test_editor (const svn_delta_editor_t **editor,
   eb->real_editor = real_editor;
   eb->real_eb = real_eb;
 
+  /* We don't implement absent_file() or absent_directory() in this
+     editor, because presumably commit would never send that. */
   ed->set_target_revision = set_target_revision;
   ed->open_root = open_root;
   ed->add_directory = add_directory;
