@@ -17,8 +17,11 @@
  */
 
 
+#include <stdlib.h>
+
 #include <apr_file_io.h>
 
+#include "svn_private_config.h"
 #include "svn_cmdline.h"
 #include "svn_error.h"
 #include "svn_opt.h"
@@ -29,10 +32,7 @@
 #include "svn_hash.h"
 #include "svn_repos.h"
 #include "svn_pools.h"
-
-/* a reasonable guess at number of nodes that would be dropped */
-#define REASONABLE_GUESS 42
-
+#include "svn_sorts.h"
 
 
 /*** Code. ***/
@@ -56,56 +56,14 @@ create_stdio_stream (svn_stream_t **stream,
                      apr_pool_t *pool)
 {
   apr_file_t *stdio_file;
-
   apr_status_t apr_err = open_fn (&stdio_file, pool);
+
   if (apr_err)
     return svn_error_wrap_apr (apr_err, "Can't open stdio file");
 
   *stream = svn_stream_from_aprfile (stdio_file, pool);
   return SVN_NO_ERROR;
 }
-#if 0
-static void
-lx_dump_2cstring_ht (apr_pool_t *p, apr_hash_t *ht)
-{
-  apr_hash_index_t *hi;
-  void *key, *val;
-
-  for (hi = apr_hash_first (p, ht); hi; hi = apr_hash_next (hi))
-    {
-      apr_hash_this (hi, &key, NULL, &val);
-      fprintf (stderr, "\n '%s' => '%s'",
-               (const char *)key, (const char *)val);
-    }
-}
-#endif
-static void
-lx_dump_2revnum_ht (apr_pool_t *p, apr_hash_t *ht)
-{
-  apr_hash_index_t *hi;
-  const void *key;
-  void *val;
-
-  for (hi = apr_hash_first (p, ht); hi; hi = apr_hash_next (hi))
-    {
-      apr_hash_this (hi, &key, NULL, &val);
-      fprintf (stderr, "\n '%" SVN_REVNUM_T_FMT "' => '%" SVN_REVNUM_T_FMT
-               "'", *((svn_revnum_t *)key), *((svn_revnum_t *)val));
-    }
-}
-
-static void
-lx_dump_cstring_ary (apr_array_header_t *array)
-{
-  int i;
-  const char *elt;
-
-  for (i= 0; i < array->nelts; i++)
-    {
-      elt = APR_ARRAY_IDX (array , i, const char *);
-      fprintf (stderr,"%s\n", elt);
-    }
-}
 
 
 /* Writes a property in dumpfile format to given stringbuf. */
@@ -139,102 +97,101 @@ write_prop_to_stringbuf (svn_stringbuf_t **strbuf,
   svn_stringbuf_appendbytes (*strbuf, "\n", 1);
 }
 
+
 /* Prefix matching function to compare node-path with set of prefixes. */
 static svn_boolean_t
 ary_prefix_match (apr_array_header_t *pfxlist, const char *path)
 {
-  int i, path_len, pfx_len;
+  int i, pfx_len, path_len = strlen (path);
   const char *pfx;
-  svn_boolean_t rval = FALSE;
 
-  path_len = strlen (path);
-
-  for (i= 0; i < pfxlist->nelts; i++)
+  for (i = 0; i < pfxlist->nelts; i++)
     {
-      pfx = APR_ARRAY_IDX (pfxlist , i, const char *);
+      pfx = APR_ARRAY_IDX (pfxlist, i, const char *);
       pfx_len = strlen (pfx);
       if (path_len < pfx_len)
         continue;
-      rval = (0 == strncmp (path, pfx, pfx_len));
-      if (rval)
-        break;
+      if (strncmp (path, pfx, pfx_len) == 0)
+        return TRUE;
     }
 
-  return rval;
+  return FALSE;
 }
-/* EXact matching function to compare node-path with set of prefixes. */
-static svn_boolean_t
-ary_exact_match (apr_array_header_t *estlist, const char *path)
-{
-  int i, path_len, est_len;
-  const char *est;
-  svn_boolean_t rval = FALSE;
 
-  path_len = strlen (path);
-
-  for (i= 0; i < estlist->nelts; i++)
-    {
-      est = APR_ARRAY_IDX (estlist , i, const char *);
-      est_len = strlen (est);
-      if (path_len != est_len)
-        continue;
-      rval = (0 == strncmp (path, est, est_len));
-      if (rval)
-        break;
-    }
-
-  return rval;
-}
-/* Note: the input stream parser calls us up with events.
-   Output of filtered dump should take place at the close-events.
-   Before that we just save supplied data in corresponding batons.
-*/
 
 
+/* Note: the input stream parser calls us up with events.  Output of
+   filtered dump should take place at the close-events.  Before that
+   we just save supplied data in corresponding batons.
+*/
+
 
 /* Filtering batons */
 
-struct parse_baton_t {
-  svn_boolean_t       do_exclude;
-  svn_boolean_t       drop_empty_revs;
-  svn_boolean_t       do_renumber_revs;
-  svn_boolean_t       preserve_revprops;
-  svn_stream_t       *in_stream;
-  svn_stream_t       *out_stream;
-  apr_int32_t         rev_drop_count;
-  apr_int32_t         node_drop_count;
+struct parse_baton_t 
+{
+  /* Command-line options values. */
+  svn_boolean_t do_exclude;
+  svn_boolean_t quiet;
+  svn_boolean_t drop_empty_revs;
+  svn_boolean_t do_renumber_revs;
+  svn_boolean_t preserve_revprops;
   apr_array_header_t *prefixes;
-  apr_array_header_t *dropped_nodes;
-  apr_hash_t         *renumber_history;
+
+  /* Input and output streams. */
+  svn_stream_t *in_stream;
+  svn_stream_t *out_stream;
+
+  /* State for the filtering process. */
+  apr_int32_t rev_drop_count;
+  apr_hash_t *dropped_nodes;
+  apr_hash_t *renumber_history;
 };
 
-struct revision_baton_t {
+struct revision_baton_t 
+{
+  /* Reference to the global parse baton. */
   struct parse_baton_t *pb;
 
-  svn_boolean_t    has_nodes;
-  svn_boolean_t    has_props;
-  svn_boolean_t    had_dropped_nodes;
-  svn_revnum_t     rev_orig;
-  svn_revnum_t     rev_actual;
+  /* Does this revision have node or prop changes? */
+  svn_boolean_t has_nodes;
+  svn_boolean_t has_props;
+
+  /* Did we drop any nodes? */
+  svn_boolean_t had_dropped_nodes;
+
+  /* The original and new (re-mapped) revision numbers. */
+  svn_revnum_t rev_orig;
+  svn_revnum_t rev_actual;
+
+  /* Pointers to dumpfile data. */
   svn_stringbuf_t *header;
-  apr_hash_t      *props;
+  apr_hash_t *props;
   svn_stringbuf_t *body;
-  svn_stream_t    *body_stream;
+  svn_stream_t *body_stream;
 };
 
-struct node_baton_t {
+struct node_baton_t 
+{
+  /* Reference to the current revision baton. */
   struct revision_baton_t *rb;
 
-  svn_boolean_t    do_skip;
-  svn_boolean_t    has_props;
-  svn_boolean_t    has_text;
-  svn_boolean_t    remove_props;
+  /* Are we skipping this node? */
+  svn_boolean_t do_skip;
+
+  /* Have we been instructed to change or remove props on, or change
+     the text of, this node? */
+  svn_boolean_t has_props;
+  svn_boolean_t remove_props;
+  svn_boolean_t has_text;
+
+  /* Pointers to dumpfile data. */
   svn_stringbuf_t *header;
   svn_stringbuf_t *props;
   svn_stringbuf_t *body;
   svn_stringbuf_t *node_path;
   svn_stringbuf_t *copyfrom_path;
-  svn_stream_t    *body_stream;
+  svn_stream_t *body_stream;
 };
 
 
@@ -295,9 +252,8 @@ new_revision_record (void **revision_baton,
 
 
   SVN_ERR (svn_stream_printf (header_stream, pool,
-                              SVN_REPOS_DUMPFILE_REVISION_NUMBER ": %"
-                              SVN_REVNUM_T_FMT "\n", rb->rev_actual));
-
+                              SVN_REPOS_DUMPFILE_REVISION_NUMBER ": %ld\n",
+                              rb->rev_actual));
 
   for (hi = apr_hash_first (pool, headers); hi; hi = apr_hash_next (hi))
     {
@@ -320,6 +276,7 @@ new_revision_record (void **revision_baton,
   return SVN_NO_ERROR;
 }
 
+
 /* UUID record here: dump it, as we do not filter them. */
 static svn_error_t *
 uuid_record (const char *uuid, void *parse_baton, apr_pool_t *pool)
@@ -330,6 +287,7 @@ uuid_record (const char *uuid, void *parse_baton, apr_pool_t *pool)
   return SVN_NO_ERROR;
 }
 
+
 /* New node here. Set up node_baton by copying headers. */
 static svn_error_t *
 new_node_record (void **node_baton,
@@ -353,15 +311,19 @@ new_node_record (void **node_baton,
 
   node_path = apr_hash_get (headers, SVN_REPOS_DUMPFILE_NODE_PATH,
                             APR_HASH_KEY_STRING);
-
-  /* shame, shame, shame ... this is NXOR. */
-  nb->do_skip = (ary_prefix_match (pb->prefixes, node_path)
-                 ? pb->do_exclude : (!pb->do_exclude));
-
   copyfrom_path = apr_hash_get (headers,
                                 SVN_REPOS_DUMPFILE_NODE_COPYFROM_PATH,
                                 APR_HASH_KEY_STRING);
 
+  /* Ensure that paths start with a leading '/'. */
+  node_path = svn_path_join ("/", node_path, pool);
+  if (copyfrom_path)
+    copyfrom_path = svn_path_join ("/", copyfrom_path, pool);
+
+  /* Shame, shame, shame ... this is NXOR. */
+  nb->do_skip = (ary_prefix_match (pb->prefixes, node_path)
+                 ? pb->do_exclude : (! pb->do_exclude));
+
   /* See if this node was copied from dropped source.  If it was,
      we have to drop this node, too.  
 
@@ -377,6 +339,7 @@ new_node_record (void **node_baton,
                            APR_HASH_KEY_STRING);
       tcl = apr_hash_get (headers, SVN_REPOS_DUMPFILE_TEXT_CONTENT_LENGTH,
                           APR_HASH_KEY_STRING);
+
       /* If there is a Text-content-length header, and the kind is
          "file", we just fallback to an add without history. */
       if (tcl && (strcmp (kind, "file") == 0))
@@ -387,22 +350,29 @@ new_node_record (void **node_baton,
                         APR_HASH_KEY_STRING, NULL);
           copyfrom_path = NULL;
         }
+      /* Else, this is either a directory or a file whose contents we
+         don't have readily available.  */
       else
         {
-          nb->do_skip = ary_exact_match (pb->dropped_nodes, copyfrom_path);
+          /* If the copy source is excluded, we can't do the right
+             thing with this copy. */
+          if (ary_prefix_match (pb->prefixes, copyfrom_path) 
+              ? pb->do_exclude : (! pb->do_exclude))
+            return svn_error_createf 
+              (SVN_ERR_INCOMPLETE_DATA, 0,
+               "Invalid copy source path '%s'", copyfrom_path);
         }
     }
 
-  /* if we're skipping the node, take note of path, discarding the rest
-     This is ugly :-(
-  */
+  /* If we're skipping the node, take note of path, discarding the
+     rest.  */
   if (nb->do_skip)
     {
-      *((const char **) apr_array_push (pb->dropped_nodes))
-        = apr_pstrdup (pb->dropped_nodes->pool, node_path);
-      
+      apr_hash_set (pb->dropped_nodes, 
+                    apr_pstrdup (apr_hash_pool_get (pb->dropped_nodes), 
+                                 node_path),
+                    APR_HASH_KEY_STRING, (void *)1);
       nb->rb->had_dropped_nodes = TRUE;
-      pb->node_drop_count ++;
     }
   else
     {
@@ -445,8 +415,8 @@ new_node_record (void **node_baton,
                 }
               SVN_ERR (svn_stream_printf
                        (header_stream, pool,
-                        SVN_REPOS_DUMPFILE_NODE_COPYFROM_REV ": %"
-                        SVN_REVNUM_T_FMT "\n", *cf_renum_rev));
+                        SVN_REPOS_DUMPFILE_NODE_COPYFROM_REV ": %ld\n",
+                        *cf_renum_rev));
               continue;
             }
 
@@ -463,6 +433,7 @@ new_node_record (void **node_baton,
   return SVN_NO_ERROR;
 }
 
+
 static svn_error_t *
 set_revision_property (void *revision_baton,
                        const char *name,
@@ -477,6 +448,7 @@ set_revision_property (void *revision_baton,
   return SVN_NO_ERROR;
 }
 
+
 static svn_error_t *
 set_node_property (void *node_baton,
                    const char *name,
@@ -493,6 +465,7 @@ set_node_property (void *node_baton,
   return SVN_NO_ERROR;
 }
 
+
 static svn_error_t *
 remove_node_props (void *node_baton)
 {
@@ -503,6 +476,7 @@ remove_node_props (void *node_baton)
   return SVN_NO_ERROR;
 }
 
+
 static svn_error_t *
 set_fulltext (svn_stream_t **stream, void *node_baton)
 {
@@ -518,7 +492,6 @@ set_fulltext (svn_stream_t **stream, void *node_baton)
 }
 
 
-
 /* Finalize node */
 static svn_error_t *
 close_node (void *node_baton)
@@ -590,6 +563,7 @@ close_revision (void *revision_baton)
   apr_hash_index_t *hi;
   apr_pool_t *hash_pool = apr_hash_pool_get (rb->props);
   svn_stringbuf_t *props = svn_stringbuf_create ("", hash_pool);
+  apr_pool_t *subpool = svn_pool_create (hash_pool);
 
   /* If this revision has no nodes left because the ones it had were
      dropped, and we are not dropping empty revisions, and we were not
@@ -617,7 +591,7 @@ close_revision (void *revision_baton)
      information to the header string.  */
   if (rb->has_props)
     {
-      for (hi = apr_hash_first (hash_pool, rb->props); 
+      for (hi = apr_hash_first (subpool, rb->props); 
            hi; 
            hi = apr_hash_next (hi))
         {
@@ -662,21 +636,25 @@ close_revision (void *revision_baton)
                                  props->data      , &(props->len)));
       SVN_ERR (svn_stream_write (rb->pb->out_stream,
                                  rb->body->data   , &(rb->body->len)));
-      fprintf (stderr, "Revision %" SVN_REVNUM_T_FMT " committed as %"
-               SVN_REVNUM_T_FMT ".\n", rb->rev_orig, rb->rev_actual);
+      if (! rb->pb->quiet)
+        SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
+                                      _("Revision %ld committed as %ld.\n"),
+                                      rb->rev_orig, rb->rev_actual));
     }
   else
     {
-      rb->pb->rev_drop_count ++;
-      fprintf (stderr, "Revision %" SVN_REVNUM_T_FMT " skipped.\n",
-               rb->rev_orig);
+      rb->pb->rev_drop_count++;
+      if (! rb->pb->quiet)
+        SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
+                                      _("Revision %ld skipped.\n"),
+                                      rb->rev_orig));
     }
+  svn_pool_destroy (subpool);
   return SVN_NO_ERROR;
 }
 
 
 /* Filtering vtable */
-
 svn_repos_parser_fns_t filtering_vtable =
   {
     new_revision_record,
@@ -691,12 +669,11 @@ svn_repos_parser_fns_t filtering_vtable =
   };
 
 
-
 
 /** Subcommands. **/
 
 static svn_opt_subcommand_t
-subcommand_help,
+  subcommand_help,
   subcommand_exclude,
   subcommand_include;
 
@@ -704,7 +681,9 @@ enum
   {
     svndumpfilter__drop_empty_revs = SVN_OPT_FIRST_LONGOPT_ID,
     svndumpfilter__renumber_revs,
-    svndumpfilter__preserve_revprops
+    svndumpfilter__preserve_revprops,
+    svndumpfilter__quiet,
+    svndumpfilter__version
   };
 
 /* Option codes and descriptions.
@@ -722,9 +701,13 @@ static const apr_getopt_option_t options_table[] =
     {NULL,            '?', 0,
      "show help on a subcommand"},
 
-    {"drop-empty-revs",   svndumpfilter__drop_empty_revs, 0,
+    {"version",            svndumpfilter__version, 0,
+     "show version information" },
+    {"quiet",              svndumpfilter__quiet, 0,
+     "Do not display filtering statistics." },
+    {"drop-empty-revs",    svndumpfilter__drop_empty_revs, 0,
      "Remove revisions emptied by filtering."},
-    {"renumber-revs",     svndumpfilter__renumber_revs, 0,
+    {"renumber-revs",      svndumpfilter__renumber_revs, 0,
      "Renumber revisions left after filtering." },
     {"preserve-revprops",  svndumpfilter__preserve_revprops, 0,
      "Don't filter revision properties." },
@@ -738,21 +721,21 @@ static const apr_getopt_option_t options_table[] =
 static const svn_opt_subcommand_desc_t cmd_table[] =
   {
     {"exclude", subcommand_exclude, {0},
-     "usage: svndumpfilter exclude PATH_PREFIX...\n\n"
-     "Filter out nodes with given prefixes from dumpstream.\n",
+     "Filter out nodes with given prefixes from dumpstream.\n"
+     "usage: svndumpfilter exclude PATH_PREFIX...\n",
      {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs,
-      svndumpfilter__preserve_revprops} },
+      svndumpfilter__preserve_revprops, svndumpfilter__quiet} },
 
     {"include", subcommand_include, {0},
-     "usage: svndumpfilter include PATH_PREFIX...\n\n"
-     "Filter out nodes without given prefixes from dumpstream.\n",
+     "Filter out nodes without given prefixes from dumpstream.\n"
+     "usage: svndumpfilter include PATH_PREFIX...\n",
      {svndumpfilter__drop_empty_revs, svndumpfilter__renumber_revs,
-      svndumpfilter__preserve_revprops} },
+      svndumpfilter__preserve_revprops, svndumpfilter__quiet} },
 
     {"help", subcommand_help, {"?", "h"},
-     "usage: svndumpfilter help [SUBCOMMAND...]\n\n"
-     "Describe the usage of this program or its subcommands.\n",
-     {0} },
+     "Describe the usage of this program or its subcommands.\n"
+     "usage: svndumpfilter help [SUBCOMMAND...]\n",
+     {svndumpfilter__version} },
 
     { NULL, NULL, {0}, NULL, {0} }
   };
@@ -763,6 +746,8 @@ struct svndumpfilter_opt_state
 {
   svn_opt_revision_t start_revision;     /* -r X[:Y] is         */
   svn_opt_revision_t end_revision;       /* not implemented.    */
+  svn_boolean_t quiet;                   /* --quiet             */
+  svn_boolean_t version;                 /* --version           */
   svn_boolean_t drop_empty_revs;         /* --drop-empty-revs   */
   svn_boolean_t help;                    /* --help or -?        */
   svn_boolean_t renumber_revs;           /* --renumber-revs     */
@@ -771,58 +756,44 @@ struct svndumpfilter_opt_state
 };
 
 
-static void
-dump_opt_state (struct svndumpfilter_opt_state *opt_state,
-                svn_boolean_t do_exclude)
-{
-  int i;
-
-  fprintf (stderr,"%s %s drop called for prefixes:\n",
-           do_exclude ? "Exclude" : "Include",
-           opt_state->drop_empty_revs ? "with" : "without");
-
-  for (i= 0; i < opt_state->prefixes->nelts; i++)
-    {
-      fprintf (stderr,"'%s'\n",
-               APR_ARRAY_IDX (opt_state->prefixes , i, const char *));
-    }
-}
-
 static svn_error_t *
 parse_baton_initialize (struct parse_baton_t **pb,
                         struct svndumpfilter_opt_state *opt_state,
                         svn_boolean_t do_exclude,
                         apr_pool_t *pool)
 {
-  dump_opt_state (opt_state, do_exclude);
-
-  *pb = apr_palloc (pool, sizeof (struct parse_baton_t));
+  struct parse_baton_t *baton = apr_palloc (pool, sizeof (*baton));
 
   /* Read the stream from STDIN.  Users can redirect a file. */
-  SVN_ERR (create_stdio_stream (&((*pb)->in_stream),
+  SVN_ERR (create_stdio_stream (&(baton->in_stream),
                                 apr_file_open_stdin, pool));
 
   /* Have the parser dump results to STDOUT. Users can redirect a file. */
-  SVN_ERR (create_stdio_stream (&((*pb)->out_stream),
+  SVN_ERR (create_stdio_stream (&(baton->out_stream),
                                 apr_file_open_stdout, pool));
 
-  (*pb)->do_exclude = do_exclude;
-  (*pb)->do_renumber_revs = opt_state->renumber_revs;
-  (*pb)->drop_empty_revs = opt_state->drop_empty_revs;
-  (*pb)->preserve_revprops = opt_state->preserve_revprops;
-  (*pb)->prefixes = opt_state->prefixes;
-  /* this may be used to shift revision numbers while filtering */
-  (*pb)->rev_drop_count = 0;
-  (*pb)->node_drop_count = 0;
+  baton->do_exclude = do_exclude;
+  baton->do_renumber_revs = opt_state->renumber_revs;
+  baton->drop_empty_revs = opt_state->drop_empty_revs;
+  baton->preserve_revprops = opt_state->preserve_revprops;
+  baton->quiet = opt_state->quiet;
+  baton->prefixes = opt_state->prefixes;
+  baton->rev_drop_count = 0; /* used to shift revnums while filtering */
+  baton->dropped_nodes = apr_hash_make (pool);
+  baton->renumber_history = apr_hash_make (pool);
 
-  (*pb)->dropped_nodes = apr_array_make (pool, REASONABLE_GUESS,
-                                         sizeof (const char *));
-  (*pb)->renumber_history = apr_hash_make (pool);
-
-  SVN_ERR (svn_stream_printf ((*pb)->out_stream, pool,
+  /* This is non-ideal: We should pass through the version of the
+   * input dumpstream.  However, our API currently doesn't allow that.
+   * Hardcoding version 2 is acceptable because:
+   *   - We currently do not accept version 3 or greater.
+   *   - Dumpstream version 1 is so ancient as to be ignorable
+   *     (0.17.x and earlier)
+   */
+  SVN_ERR (svn_stream_printf (baton->out_stream, pool,
                               SVN_REPOS_DUMPFILE_MAGIC_HEADER ": %d\n\n",
-                              SVN_REPOS_DUMPFILE_FORMAT_VERSION));
+                              2));
 
+  *pb = baton;
   return SVN_NO_ERROR;
 }
 
@@ -830,6 +801,7 @@ parse_baton_initialize (struct parse_baton_t **pb,
 static svn_error_t *
 subcommand_help (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 {
+  struct svndumpfilter_opt_state *opt_state = baton;
   const char *header =
     "general usage: svndumpfilter SUBCOMMAND [ARGS & OPTIONS ...]\n"
     "Type \"svndumpfilter help <subcommand>\" for help on a "
@@ -837,45 +809,167 @@ subcommand_help (apr_getopt_t *os, void *baton, apr_pool_t *pool)
     "\n"
     "Available subcommands:\n";
 
-  SVN_ERR (svn_opt_print_help (os, "svndumpfilter", FALSE, FALSE, NULL,
+  SVN_ERR (svn_opt_print_help (os, "svndumpfilter",
+                               opt_state ? opt_state->version : FALSE,
+                               FALSE, NULL,
                                header, cmd_table, options_table, NULL,
                                pool));
 
   return SVN_NO_ERROR;
 }
 
+
+/* Version compatibility check */
+static svn_error_t *
+check_lib_versions (void)
+{
+  static const svn_version_checklist_t checklist[] =
+    {
+      { "svn_subr",  svn_subr_version },
+      { "svn_repos", svn_repos_version },
+      { "svn_fs",    svn_fs_version },
+      { "svn_delta", svn_delta_version },
+      { NULL, NULL }
+    };
+
+   SVN_VERSION_DEFINE (my_version);
+   return svn_ver_check_list (&my_version, checklist);
+}
+
+
+/* Do the real work of filtering. */
+static svn_error_t *
+do_filter (apr_getopt_t *os, 
+           void *baton, 
+           svn_boolean_t do_exclude,
+           apr_pool_t *pool)
+{
+  struct svndumpfilter_opt_state *opt_state = baton;
+  struct parse_baton_t *pb;
+  apr_hash_index_t *hi;
+  apr_array_header_t *keys;
+  const void *key;
+  int i, num_keys;
+
+  if (! opt_state->quiet)
+    {
+      apr_pool_t *subpool = svn_pool_create (pool);
+
+      SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
+                                    "%s %sprefixes:\n",
+                                    do_exclude ? "Excluding" : "Including",
+                                    opt_state->drop_empty_revs 
+                                    ? "(and dropping empty revisions for) "
+                                    : ""));
+
+      for (i = 0; i < opt_state->prefixes->nelts; i++)
+        {
+          svn_pool_clear (subpool);
+          SVN_ERR (svn_cmdline_fprintf
+                   (stderr, subpool, "   '%s'\n",
+                    APR_ARRAY_IDX (opt_state->prefixes, i, const char *)));
+        }
+
+      SVN_ERR (svn_cmdline_fputs ("\n", stderr, subpool));
+      svn_pool_destroy (subpool);
+    }
+
+  SVN_ERR (parse_baton_initialize (&pb, opt_state, do_exclude, pool));
+  SVN_ERR (svn_repos_parse_dumpstream (pb->in_stream, &filtering_vtable, pb,
+                                       NULL, NULL, pool));
+
+  /* The rest of this is just reporting.  If we aren't reporting, get
+     outta here. */
+  if (opt_state->quiet)
+    return SVN_NO_ERROR;
+
+  SVN_ERR (svn_cmdline_fputs ("\n", stderr, pool));
+
+  if (pb->rev_drop_count)
+    SVN_ERR (svn_cmdline_fprintf (stderr, pool,
+                                  _("Dropped %d revision(s).\n\n"),
+                                  pb->rev_drop_count));
+
+  if (pb->do_renumber_revs)
+    {
+      apr_pool_t *subpool = svn_pool_create (pool);
+      SVN_ERR (svn_cmdline_fputs (_("Revisions renumbered as follows:\n"),
+                                  stderr, subpool));
+
+      /* Get the keys of the hash, sort them, then print the hash keys
+         and values, sorted by keys. */
+      num_keys = apr_hash_count (pb->renumber_history);
+      keys = apr_array_make (pool, num_keys + 1, sizeof (svn_revnum_t));
+      for (hi = apr_hash_first (pool, pb->renumber_history); 
+           hi; 
+           hi = apr_hash_next (hi))
+        {
+          apr_hash_this (hi, &key, NULL, NULL);
+          APR_ARRAY_PUSH (keys, svn_revnum_t) = *((svn_revnum_t *) key);
+        }
+      qsort (keys->elts, keys->nelts, 
+             keys->elt_size, svn_sort_compare_revisions);
+      for (i = 0; i < keys->nelts; i++)
+        {
+          svn_revnum_t this_key, this_val;
+
+          svn_pool_clear (subpool);
+          this_key = APR_ARRAY_IDX (keys, i, svn_revnum_t);
+          this_val = 
+            *((svn_revnum_t *)apr_hash_get (pb->renumber_history,
+                                            &this_key,
+                                            sizeof (this_key)));
+          if (this_val == SVN_INVALID_REVNUM)
+            SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
+                                          _("   %ld => (dropped)\n"),
+                                          this_key));
+          else
+            SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
+                                          _("   %ld => %ld\n"),
+                                          this_key, this_val));
+        }
+      SVN_ERR (svn_cmdline_fputs ("\n", stderr, subpool));
+      svn_pool_destroy (subpool);
+    }
+
+  if (apr_hash_count (pb->dropped_nodes))
+    {
+      apr_pool_t *subpool = svn_pool_create (pool);
+      SVN_ERR (svn_cmdline_fprintf (stderr, subpool,
+                                    _("Dropped %d node(s):\n"), 
+                                    apr_hash_count (pb->dropped_nodes)));
+
+      /* Get the keys of the hash, sort them, then print the hash keys
+         and values, sorted by keys. */
+      num_keys = apr_hash_count (pb->dropped_nodes);
+      keys = apr_array_make (pool, num_keys + 1, sizeof (const char *));
+      for (hi = apr_hash_first (pool, pb->dropped_nodes);
+           hi; 
+           hi = apr_hash_next (hi))
+        {
+          apr_hash_this (hi, &key, NULL, NULL);
+          APR_ARRAY_PUSH (keys, const char *) = key;
+        }
+      qsort (keys->elts, keys->nelts, keys->elt_size, svn_sort_compare_paths);
+      for (i = 0; i < keys->nelts; i++)
+        {
+          svn_pool_clear (subpool);
+          SVN_ERR (svn_cmdline_fprintf
+                   (stderr, subpool, "   '%s'\n", 
+                    (const char *)APR_ARRAY_IDX (keys, i, const char *)));
+        }
+      SVN_ERR (svn_cmdline_fputs ("\n", stderr, subpool));
+      svn_pool_destroy (subpool);
+    }
+
+  return SVN_NO_ERROR;
+}
+
 /* This implements `exclude' subcommand. */
 static svn_error_t *
 subcommand_exclude (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 {
-  struct svndumpfilter_opt_state *opt_state = baton;
-  struct parse_baton_t *pb;
-
-  SVN_ERR (parse_baton_initialize (&pb, opt_state, TRUE, pool));
-
-  SVN_ERR (svn_repos_parse_dumpstream (pb->in_stream,
-                                       &filtering_vtable, pb,
-                                       NULL, NULL, pool));
-
-  fprintf (stderr, "\n Dropped %d revisions, %d nodes\n",
-           pb->rev_drop_count, pb->node_drop_count);
-
-  if (pb->do_renumber_revs)
-    {
-      fprintf (stderr, "\nRenumber history:\n");
-      lx_dump_2revnum_ht (pool, pb->renumber_history);
-    }
-
-  if (pb->node_drop_count > 0)
-    {
-      fprintf (stderr, "\n\nDropped nodes list:\n");
-      lx_dump_cstring_ary (pb->dropped_nodes);
-    }
-  else
-    {
-      fprintf (stderr, "\n\nNo nodes dropped.\n");
-    }
-  return SVN_NO_ERROR;
+  return do_filter (os, baton, TRUE, pool);
 }
 
 
@@ -883,35 +977,7 @@ subcommand_exclude (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 static svn_error_t *
 subcommand_include (apr_getopt_t *os, void *baton, apr_pool_t *pool)
 {
-  struct svndumpfilter_opt_state *opt_state = baton;
-  struct parse_baton_t *pb;
-
-  SVN_ERR (parse_baton_initialize (&pb, opt_state, FALSE, pool));
-
-  SVN_ERR (svn_repos_parse_dumpstream (pb->in_stream,
-                                       &filtering_vtable, pb,
-                                       NULL, NULL, pool));
-
-  fprintf (stderr, "\n Dropped %d revisions, %d nodes\n",
-           pb->rev_drop_count, pb->node_drop_count);
-
-  if (pb->do_renumber_revs)
-    {
-      fprintf (stderr, "\nRenumber history:\n");
-      lx_dump_2revnum_ht (pool, pb->renumber_history);
-    }
-
-  if (pb->node_drop_count > 0)
-    {
-      fprintf (stderr, "\n\nDropped nodes list:\n");
-      lx_dump_cstring_ary (pb->dropped_nodes);
-    }
-  else
-    {
-      fprintf (stderr, "\n\nNo nodes dropped.\n");
-    }
-
-  return SVN_NO_ERROR;
+  return do_filter (os, baton, FALSE, pool);
 }
 
 
@@ -949,6 +1015,16 @@ main (int argc, const char * const *argv)
   pool = svn_pool_create_ex (NULL, allocator);
   apr_allocator_owner_set (allocator, pool);
 		  
+  /* Check library versions */
+  err = check_lib_versions ();
+  if (err)
+    {
+      svn_handle_error (err, stderr, FALSE);
+      svn_error_clear (err);
+      svn_pool_destroy (pool);
+      return EXIT_FAILURE;
+    }
+
   if (argc <= 1)
     {
       subcommand_help (NULL, NULL, pool);
@@ -989,6 +1065,12 @@ main (int argc, const char * const *argv)
         case '?':
           opt_state.help = TRUE;
           break;
+        case svndumpfilter__version:
+          opt_state.version = TRUE;
+          opt_state.help = TRUE;
+        case svndumpfilter__quiet:
+          opt_state.quiet = TRUE;
+          break;
         case svndumpfilter__drop_empty_revs:
           opt_state.drop_empty_revs = TRUE;
           break;
@@ -1020,7 +1102,8 @@ main (int argc, const char * const *argv)
     {
       if (os->ind >= os->argc)
         {
-          fprintf (stderr, "subcommand argument required\n");
+          svn_error_clear (svn_cmdline_fprintf
+                           (stderr, pool, "subcommand argument required\n"));
           subcommand_help (NULL, NULL, pool);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
@@ -1031,7 +1114,19 @@ main (int argc, const char * const *argv)
           subcommand = svn_opt_get_canonical_subcommand (cmd_table, first_arg);
           if (subcommand == NULL)
             {
-              fprintf (stderr, "unknown command: '%s'\n", first_arg);
+              const char* first_arg_utf8;
+              if ((err = svn_utf_cstring_to_utf8 (&first_arg_utf8, first_arg,
+                                                  pool)))
+                {
+                  svn_handle_error (err, stderr, FALSE);
+                  svn_pool_destroy (pool);
+                  svn_error_clear (err);
+                  return EXIT_FAILURE;
+                }
+                
+              svn_error_clear (svn_cmdline_fprintf (stderr, pool,
+                                                    "unknown command: '%s'\n",
+                                                    first_arg_utf8));
               subcommand_help (NULL, NULL, pool);
               svn_pool_destroy (pool);
               return EXIT_FAILURE;
@@ -1045,32 +1140,30 @@ main (int argc, const char * const *argv)
 
   if (subcommand->cmd_func != subcommand_help)
     {
-      if (os->ind < os->argc)
+      if (os->ind >= os->argc)
         {
-          opt_state.prefixes = apr_array_make (pool,
-                                               os->argc - os->ind,
-                                               sizeof (const char *));
-          for (i = os->ind ; i< os->argc; i++)
-            {
-              const char *prefix_utf8;
-              SVN_INT_ERR (svn_utf_cstring_to_utf8 (&prefix_utf8,
-                                                    os->argv[i],
-                                                    pool));
-              *(const char **)apr_array_push (opt_state.prefixes)
-                = svn_path_internal_style (prefix_utf8, pool);
-            }
-        }
-      else
-        {
-          fprintf (stderr,
-                   "\nError: no prefixes supplied.\n");
-          svn_opt_subcommand_help (subcommand->name,
-                                   cmd_table,
-                                   options_table,
-                                   pool);
+          svn_error_clear (svn_cmdline_fprintf
+                           (stderr, pool,
+                            _("\nError: no prefixes supplied.\n")));
+          svn_opt_subcommand_help (subcommand->name, cmd_table,
+                                   options_table, pool);
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
         }
+
+      opt_state.prefixes = apr_array_make (pool, os->argc - os->ind,
+                                           sizeof (const char *));
+      for (i = os->ind ; i< os->argc; i++)
+        {
+          const char *prefix;
+
+          /* Ensure that each prefix is UTF8-encoded, in internal
+             style, and absolute. */
+          SVN_INT_ERR (svn_utf_cstring_to_utf8 (&prefix, os->argv[i], pool));
+          prefix = svn_path_internal_style (prefix, pool);
+          prefix = svn_path_join ("/", prefix, pool);
+          APR_ARRAY_PUSH (opt_state.prefixes, const char *) = prefix;
+        }
     }
 
 
@@ -1092,10 +1185,11 @@ main (int argc, const char * const *argv)
           const apr_getopt_option_t *badopt =
             svn_opt_get_option_from_code (opt_id, options_table);
           svn_opt_format_option (&optstr, badopt, FALSE, pool);
-          fprintf (stderr,
-                   "subcommand '%s' doesn't accept option '%s'\n"
-                   "Type 'svndumpfilter help %s' for usage.\n",
-                   subcommand->name, optstr, subcommand->name);
+          svn_error_clear (svn_cmdline_fprintf
+                           (stderr, pool,
+                            _("subcommand '%s' doesn't accept option '%s'\n"
+                              "Type 'svndumpfilter help %s' for usage.\n"),
+                            subcommand->name, optstr, subcommand->name));
           svn_pool_destroy (pool);
           return EXIT_FAILURE;
         }
@@ -1112,13 +1206,18 @@ main (int argc, const char * const *argv)
                                    options_table, pool);
         }
       else
-        svn_handle_error (err, stderr, 0);
+        {
+          svn_handle_error (err, stderr, 0);
+        }
       svn_pool_destroy (pool);
       return EXIT_FAILURE;
     }
   else
     {
       svn_pool_destroy (pool);
+
+      /* Flush stdout, making sure the user will see any print errors. */
+      SVN_INT_ERR (svn_cmdline_fflush (stdout));
       return EXIT_SUCCESS;
     }
 }
