@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2003 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2004 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -21,7 +21,7 @@ import shutil, stat, string, sys, re, os.path
 
 # Our testing module
 import svntest
-from svntest import wc
+from svntest import wc, SVNAnyOutput
 
 # (abbreviation)
 Skip = svntest.testcase.Skip
@@ -52,20 +52,19 @@ def basic_checkout(sbox):
   "basic checkout of a wc"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Checkout of a different URL into a working copy fails
   A_url = svntest.main.current_repo_url + '/A'
-  stdout_lines, stderr_lines = svntest.main.run_svn ("Obstructed update",
-                                                     'co', A_url,
-                                                     '--username',
-                                                     svntest.main.wc_author,
-                                                     '--password',
-                                                     svntest.main.wc_passwd,
-                                                     wc_dir)
-  if not stderr_lines:
-    raise svntest.Failure
+  svntest.actions.run_and_verify_svn("No error where some expected",
+                                      None, SVNAnyOutput,
+                                     # "Obstructed update",
+                                     'co', A_url,
+                                     '--username',
+                                     svntest.main.wc_author,
+                                     '--password',
+                                     svntest.main.wc_passwd,
+                                     wc_dir)
 
   # Make some changes to the working copy
   mu_path = os.path.join(wc_dir, 'A', 'mu')
@@ -76,7 +75,6 @@ def basic_checkout(sbox):
 
   svntest.actions.run_and_verify_svn(None, None, [], 'rm', G_path)
 
-  extra_files = ['lambda']
   expected_output = svntest.actions.get_virginal_state(wc_dir, 1)
   expected_output.tweak('A/mu', status='M ')
   expected_output.tweak('A/B/lambda', status='! ')
@@ -85,12 +83,7 @@ def basic_checkout(sbox):
                         'A/D/G/rho',
                         'A/D/G/tau', status='D ')
   
-  svntest.actions.run_and_verify_status(wc_dir, expected_output,
-                                        None, None,
-                                        expect_extra_files, extra_files)
-  if len(extra_files) != 0:
-    print "Status check 1 failed"
-    raise svntest.Failure
+  svntest.actions.run_and_verify_status(wc_dir, expected_output)
 
   # Repeat checkout of original URL into working copy with modifications
   url = svntest.main.current_repo_url
@@ -114,7 +107,6 @@ def basic_status(sbox):
   "basic status command"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Created expected output tree for 'svn status'
@@ -128,7 +120,6 @@ def basic_commit(sbox):
   "basic commit command"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Make a couple of local mods to files
@@ -164,7 +155,6 @@ def basic_update(sbox):
   "basic update command"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Make a backup copy of the working copy
@@ -218,6 +208,39 @@ def basic_update(sbox):
                                         expected_disk,
                                         expected_status)
 
+#----------------------------------------------------------------------
+def basic_mkdir_url(sbox):
+  "basic mkdir URL"
+
+  sbox.build()
+
+  Y_url = svntest.main.current_repo_url + '/Y'
+  Y_Z_url = svntest.main.current_repo_url + '/Y/Z'
+
+  svntest.actions.run_and_verify_svn("mkdir URL URL/subdir",
+                                     ["\n", "Committed revision 2.\n"], [],
+                                     'mkdir', '-m', 'log_msg', Y_url, Y_Z_url)
+
+  expected_output = wc.State(sbox.wc_dir, {
+    'Y'   : Item(status='A '),
+    'Y/Z' : Item(status='A '),
+    })
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.add({
+    'Y'   : Item(),
+    'Y/Z' : Item()
+    })
+  expected_status = svntest.actions.get_virginal_state(sbox.wc_dir, 2)
+  expected_status.add({
+    'Y'   : Item(status='  ', wc_rev=2, repos_rev=2),
+    'Y/Z' : Item(status='  ', wc_rev=2, repos_rev=2)
+    })
+
+  svntest.actions.run_and_verify_update(sbox.wc_dir,
+                                        expected_output,
+                                        expected_disk,
+                                        expected_status)
+
 #----------------------------------------------------------------------
 def basic_corruption(sbox):
   "basic corruption detection"
@@ -238,7 +261,6 @@ def basic_corruption(sbox):
   ## Here we go...
 
   sbox.build()
-  
   wc_dir = sbox.wc_dir
 
   # Make the "other" working copy
@@ -276,7 +298,7 @@ def basic_corruption(sbox):
 
   # This commit should fail due to text base corruption.
   svntest.actions.run_and_verify_commit (wc_dir, expected_output,
-                                            expected_status, "checksum",
+                                            expected_status, "svn: Checksum",
                                             None, None, None, None, wc_dir)
 
   # Restore the uncorrupted text base.
@@ -325,7 +347,7 @@ def basic_corruption(sbox):
                                         expected_output,
                                         expected_disk,
                                         expected_status,
-                                        "checksum", other_wc)
+                                        "svn: Checksum", other_wc)
   
   # Restore the uncorrupted text base.
   os.chmod (tb_dir_path, 0777)
@@ -347,7 +369,6 @@ def basic_merging_update(sbox):
   "receiving text merges as part of an update"
 
   sbox.build()
-  
   wc_dir = sbox.wc_dir
   
   # First change the greek tree to make two files 10 lines long
@@ -458,7 +479,6 @@ def basic_conflict(sbox):
   "basic conflict creation and resolution"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Make a backup copy of the working copy
@@ -549,10 +569,10 @@ Original appended text for rho>>>>>>> .r2
     raise svntest.Failure
 
   # So now mu and rho are both in a "conflicted" state.  Run 'svn
-  # resolve' on them.
+  # resolved' on them.
 
-  svntest.actions.run_and_verify_svn("Resolve command", None, [],
-                                     'resolve',
+  svntest.actions.run_and_verify_svn("Resolved command", None, [],
+                                     'resolved',
                                      mu_path_backup,
                                      rho_path_backup)
 
@@ -571,7 +591,6 @@ def basic_cleanup(sbox):
   "basic cleanup command"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Lock some directories.
@@ -604,7 +623,6 @@ def basic_revert(sbox):
   "basic revert command"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Modify some files.
@@ -728,7 +746,6 @@ def basic_switch(sbox):
   "basic switch command"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   ### Switch the file `iota' to `A/D/gamma'.
@@ -809,11 +826,11 @@ def basic_switch(sbox):
                                         expected_disk,
                                         expected_status)
 
-
 #----------------------------------------------------------------------
 
 def verify_file_deleted(message, path):
-  try: open(path, 'r')
+  try:
+    open(path, 'r')
   except IOError:
     return
   if message is not None:
@@ -821,11 +838,12 @@ def verify_file_deleted(message, path):
   ###TODO We should raise a less generic error here. which?
   raise Failure
   
-
 def can_cd_to_dir(path):
-  current_dir = os.getcwd();
-  try: os.chdir(path)
-  except OSError: return 0
+  current_dir = os.getcwd()
+  try:
+    os.chdir(path)
+  except OSError:
+    return 0
   os.chdir(current_dir)
   return 1
   
@@ -833,7 +851,6 @@ def basic_delete(sbox):
   "basic delete command"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # modify text of chi
@@ -886,31 +903,31 @@ def basic_delete(sbox):
   svntest.actions.run_and_verify_status(wc_dir, expected_output)
 
   # 'svn rm' that should fail
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', chi_path)
 
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', chi_parent_path)
   
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', rho_path)
 
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', rho_parent_path)
   
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', F_path)
 
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', F_parent_path)
   
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', sigma_path)
 
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', sigma_parent_path)
 
-  svntest.actions.run_and_verify_svn(None, None, svntest.SVNAnyOutput,
+  svntest.actions.run_and_verify_svn(None, None, SVNAnyOutput,
                                      'rm', X_path)
 
   # check status has not changed
@@ -1033,7 +1050,6 @@ def basic_checkout_deleted(sbox):
   "checkout a path no longer in HEAD"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # Delete A/D and commit.
@@ -1057,8 +1073,7 @@ def basic_checkout_deleted(sbox):
                                         wc_dir)
 
   # Now try to checkout revision 1 of A/D.
-  url = svntest.main.test_area_url + '/' + svntest.main.current_repo_dir + \
-        '/A/D'
+  url = svntest.main.current_repo_url + '/A/D'
   wc2 = os.path.join (sbox.wc_dir, 'new_D')
   svntest.actions.run_and_verify_svn("error checking out r1 of A/D",
                                      None, [], 'co', '-r', '1',
@@ -1077,7 +1092,7 @@ def basic_node_kind_change(sbox):
   "attempt to change node kind"
 
   sbox.build()
-  wc_dir = sbox.wc_dir;
+  wc_dir = sbox.wc_dir
   
   # Schedule a file for deletion
   gamma_path = os.path.join(wc_dir, 'A', 'D', 'gamma')
@@ -1090,7 +1105,7 @@ def basic_node_kind_change(sbox):
 
   # Try and fail to create a directory (file scheduled for deletion)
   svntest.actions.run_and_verify_svn('Cannot change node kind',
-                                     None, svntest.SVNAnyOutput,
+                                     None, SVNAnyOutput,
                                      'mkdir', gamma_path)
 
   # Status is unchanged
@@ -1110,7 +1125,7 @@ def basic_node_kind_change(sbox):
 
   # Try and fail to create a directory (file deleted)
   svntest.actions.run_and_verify_svn('Cannot change node kind',
-                                     None, svntest.SVNAnyOutput,
+                                     None, SVNAnyOutput,
                                      'mkdir', gamma_path)
 
   # Status is unchanged
@@ -1134,7 +1149,6 @@ def basic_import(sbox):
   "basic import of single new file"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # create a new directory with files of various permissions
@@ -1143,12 +1157,12 @@ def basic_import(sbox):
   svntest.main.file_append(new_path, "some text")
 
   # import new files into repository
-  url = svntest.main.current_repo_url
+  url = svntest.main.current_repo_url + "/dirA/dirB/new_file"
   output, errput =   svntest.actions.run_and_verify_svn(
     'Cannot change node kind', None, [], 'import',
     '--username', svntest.main.wc_author,
     '--password', svntest.main.wc_passwd,
-    '-m', 'Log message for new import', url, new_path, 'new_file')
+    '-m', 'Log message for new import', new_path, url)
 
   lastline = string.strip(output.pop())
   cm = re.compile ("(Committed|Imported) revision [0-9]+.")
@@ -1163,19 +1177,23 @@ def basic_import(sbox):
   # Create expected disk tree for the update (disregarding props)
   expected_disk = svntest.main.greek_state.copy()
   expected_disk.add({
-    'new_file' : Item('some text'),
+    'dirA/dirB/new_file' : Item('some text'),
     })
 
   # Create expected status tree for the update (disregarding props).
   # Newly imported file should be at revision 2.
   expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
   expected_status.add({
-    'new_file' : Item(status='  ', wc_rev=2, repos_rev=2),
+    'dirA'                : Item(status='  ', wc_rev=2, repos_rev=2),
+    'dirA/dirB'           : Item(status='  ', wc_rev=2, repos_rev=2),
+    'dirA/dirB/new_file'  : Item(status='  ', wc_rev=2, repos_rev=2),
     })
 
   # Create expected output tree for the update.
   expected_output = svntest.wc.State(wc_dir, {
-    'new_file' : Item(status='A '),
+    'dirA'               : Item(status='A '),
+    'dirA/dirB'          : Item(status='A '),
+    'dirA/dirB/new_file' : Item(status='A '),
   })
 
   # do update and check three ways
@@ -1193,7 +1211,6 @@ def basic_import_executable(sbox):
   "basic import of executable files"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   # create a new directory with files of various permissions
@@ -1221,7 +1238,7 @@ def basic_import_executable(sbox):
     None, None, [], 'import',
     '--username', svntest.main.wc_author,
     '--password', svntest.main.wc_passwd,
-    '-m', 'Log message for new import', url, xt_path)
+    '-m', 'Log message for new import', xt_path, url)
 
   lastline = string.strip(output.pop())
   cm = re.compile ("(Committed|Imported) revision [0-9]+.")
@@ -1231,7 +1248,7 @@ def basic_import_executable(sbox):
     raise svntest.Failure
 
   # remove (uncontrolled) local files
-  shutil.rmtree(xt_path)
+  svntest.main.safe_rmtree(xt_path)
 
   # Create expected disk tree for the update (disregarding props)
   expected_disk = svntest.main.greek_state.copy()
@@ -1277,7 +1294,6 @@ def basic_cat(sbox):
   "basic cat of files"
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   mu_path = os.path.join(wc_dir, 'A', 'mu')
@@ -1328,6 +1344,13 @@ def basic_ls(sbox):
                                      '--password', svntest.main.wc_passwd,
                                      os.path.join(wc_dir, 'A'))
 
+  svntest.actions.run_and_verify_svn('ls working copy directory with -r BASE',
+                                     ['B/\n', 'C/\n', 'D/\n', 'mu\n'],
+                                     None, 'ls', '-r', 'BASE',
+                                     '--username', svntest.main.wc_author,
+                                     '--password', svntest.main.wc_passwd,
+                                     os.path.join(wc_dir, 'A'))
+
   svntest.actions.run_and_verify_svn('ls a single file',
                                      ['mu\n'],
                                      None, 'ls',
@@ -1366,7 +1389,7 @@ def nonexistent_repository(sbox):
   # the root directory, the test could fail, and that's just too bad :-). 
 
   output, errput = svntest.actions.run_and_verify_svn(
-    None, None, svntest.SVNAnyOutput,
+    None, None, SVNAnyOutput,
     'log', 'file:///nonexistent_path')
 
   for line in errput:
@@ -1384,8 +1407,8 @@ def basic_auth_cache(sbox):
   "basic auth caching"
 
   sbox.build()
-
   wc_dir         = sbox.wc_dir
+  
   repo_dir       = sbox.repo_dir
   repo_url       = sbox.repo_url
 
@@ -1419,7 +1442,7 @@ def basic_auth_cache(sbox):
 
 #----------------------------------------------------------------------
 def basic_add_ignores(sbox):
-  'ignored files in the added directory should not be added'
+  'ignored files in added dirs should not be added'
 
   # The bug was that
   #
@@ -1432,7 +1455,6 @@ def basic_add_ignores(sbox):
   # svn_wc_is_ignored function.
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   dir_path = os.path.join(wc_dir, 'dir')
@@ -1443,22 +1465,21 @@ def basic_add_ignores(sbox):
   open(foo_c_path, 'w')
   open(foo_o_path, 'w')
 
-  (output, errput) = svntest.main.run_svn (None, 'add', dir_path)
-
-  if not output:
-    raise svntest.main.SVNUnexpectedOutput
+  output, err = svntest.actions.run_and_verify_svn(
+    "No output where some expected", SVNAnyOutput, None,
+    'add', dir_path)
 
   for line in output:
     # If we see foo.o in the add output, fail the test.
     if re.match(r'^A\s+.*foo.o$', line):
-      raise svntest.main.SVNUnexpectedOutput
+      raise svntest.actions.SVNUnexpectedOutput
 
   # Else never matched the unwanted output, so the test passed.
 
 
 #----------------------------------------------------------------------
 def basic_import_ignores(sbox):
-  'ignored files in the imported directory should not be imported'
+  'do not import ignored files in imported dirs'
 
   # The bug was that
   #
@@ -1471,7 +1492,6 @@ def basic_import_ignores(sbox):
   # svn_wc_is_ignored function.
 
   sbox.build()
-
   wc_dir = sbox.wc_dir
 
   dir_path = os.path.join(wc_dir, 'dir')
@@ -1483,25 +1503,24 @@ def basic_import_ignores(sbox):
   open(foo_o_path, 'w')
 
   # import new dir into repository
-  url = svntest.main.current_repo_url
-
+  url = svntest.main.current_repo_url + '/dir'
 
   output, errput = svntest.actions.run_and_verify_svn(
     None, None, [], 'import',
     '--username', svntest.main.wc_author,
     '--password', svntest.main.wc_passwd,
     '-m', 'Log message for new import',
-    url, dir_path, 'dir')
+    dir_path, url)
 
   lastline = string.strip(output.pop())
   cm = re.compile ("(Committed|Imported) revision [0-9]+.")
   match = cm.search (lastline)
   if not match:
     ### we should raise a less generic error here. which?
-    raise svntest.SVNUnexpectedOutput
+    raise svntest.actions.SVNUnexpectedOutput
 
   # remove (uncontrolled) local dir
-  shutil.rmtree(dir_path)
+  svntest.main.safe_rmtree(dir_path)
 
   # Create expected disk tree for the update (disregarding props)
   expected_disk = svntest.main.greek_state.copy()
@@ -1531,8 +1550,142 @@ def basic_import_ignores(sbox):
                                         None, None, None,
                                         None, None, 1)
 
+#----------------------------------------------------------------------
+def uri_syntax(sbox):
+  'make sure URI syntaxes are parsed correctly'
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+
+  # Revision 6638 made 'svn co http://host' seg fault, this tests the fix.
+  svntest.main.safe_rmtree(wc_dir)
+  url = svntest.main.current_repo_url
+  schema = url[:string.find(url, ":")]
+  url = schema + "://some_nonexistent_host_with_no_trailing_slash"
+  svntest.actions.run_and_verify_svn("No error where one expected",
+                                     None, SVNAnyOutput,
+                                     'co', url, wc_dir)
+
+  # Different RA layers give different errors for failed checkouts;
+  # for us, it's only important to know that it _did_ error (as
+  # opposed to segfaulting), so we don't examine the error text.
 
 #----------------------------------------------------------------------
+def basic_checkout_file(sbox):
+  "trying to check out a file should fail"
+
+  sbox.build()
+
+  iota_url = svntest.main.current_repo_url + '/iota'
+
+  output, errput = svntest.main.run_svn(1, 'co', iota_url)
+
+  for line in errput:
+    if string.find(line, "refers to a file") != -1:
+      break
+  else:
+    raise svntest.Failure
+
+#----------------------------------------------------------------------
+def basic_history(sbox):
+  "verify that 'svn cat' traces renames"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  rho_path   = os.path.join(wc_dir, 'A', 'D', 'G', 'rho')
+  pi_path    = os.path.join(wc_dir, 'A', 'D', 'G', 'pi')
+  bloo_path  = os.path.join(wc_dir, 'A', 'D', 'G', 'bloo')
+
+  # rename rho to bloo. commit r2.
+  svntest.main.run_svn(None, 'mv', rho_path, bloo_path)
+
+  expected_output = svntest.wc.State(wc_dir, {
+    'A/D/G/rho' : Item(verb='Deleting'),
+    'A/D/G/bloo' : Item(verb='Adding')
+    })
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
+  expected_status.tweak(wc_rev=1)
+  expected_status.remove('A/D/G/rho');
+  expected_status.add({ 'A/D/G/bloo' :
+                        Item(wc_rev=2, repos_rev=2, status='  ') })
+
+  svntest.actions.run_and_verify_commit (wc_dir,
+                                         expected_output,
+                                         expected_status,
+                                         None,
+                                         None, None,
+                                         None, None,
+                                         wc_dir)
+  
+  # rename pi to rho.  commit r3.
+  svntest.main.run_svn(None, 'mv', pi_path, rho_path)
+
+  # svn cat -r1 rho  --> should show pi's contents.
+  svntest.actions.run_and_verify_svn (None,
+                                      [ "This is the file 'pi'."], None,
+                                      'cat',  '-r', '1', rho_path)
+  
+  expected_output = svntest.wc.State(wc_dir, {
+    'A/D/G/pi' : Item(verb='Deleting'),
+    'A/D/G/rho' : Item(verb='Adding')
+    })
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 3)
+  expected_status.tweak(wc_rev=1)
+  expected_status.remove('A/D/G/pi');
+  expected_status.tweak('A/D/G/rho', wc_rev=3)
+  expected_status.add({ 'A/D/G/bloo' :
+                        Item(wc_rev=2, repos_rev=3, status='  ') })
+
+  svntest.actions.run_and_verify_commit (wc_dir,
+                                         expected_output,
+                                         expected_status,
+                                         None,
+                                         None, None,
+                                         None, None,
+                                         wc_dir)
+
+  # update whole wc to HEAD
+  expected_output = svntest.wc.State(wc_dir, { }) # no output
+  expected_status.tweak(wc_rev=3)
+  expected_disk = svntest.main.greek_state.copy()
+  expected_disk.remove('A/D/G/pi', 'A/D/G/rho')
+  expected_disk.add({
+    'A/D/G/rho' : Item("This is the file 'pi'."),
+    })
+  expected_disk.add({
+    'A/D/G/bloo' : Item("This is the file 'rho'."),
+    })
+  svntest.actions.run_and_verify_update(wc_dir,
+                                        expected_output,
+                                        expected_disk,
+                                        expected_status)  
+
+  # 'svn cat bloo' --> should show rho's contents.
+  svntest.actions.run_and_verify_svn (None,
+                                      [ "This is the file 'rho'."], None,
+                                      'cat',  bloo_path)
+  
+  # svn cat -r1 bloo --> should still show rho's contents.
+  svntest.actions.run_and_verify_svn (None,
+                                      [ "This is the file 'rho'."], None,
+                                      'cat',  '-r', '1', bloo_path)
+
+  # svn cat -r1 rho  --> should show pi's contents.
+  svntest.actions.run_and_verify_svn (None,
+                                      [ "This is the file 'pi'."], None,
+                                      'cat',  '-r', '1', rho_path)
+  
+  # svn up -r1
+  svntest.actions.run_and_verify_svn(None, None, [], 'up', '-r', '1', wc_dir)
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 1)
+  expected_status.tweak(repos_rev=3)
+  svntest.actions.run_and_verify_status(wc_dir, expected_status)
+
+  # svn cat -rHEAD rho --> should see 'unrelated object' error.
+  svntest.actions.run_and_verify_svn ("unrelated object",
+                                      None, SVNAnyOutput,
+                                      'cat',  '-r', 'HEAD', rho_path)
+
 
 ########################################################################
 # Run the tests
@@ -1544,6 +1697,7 @@ test_list = [ None,
               basic_status,
               basic_commit,
               basic_update,
+              basic_mkdir_url,
               basic_corruption,
               basic_merging_update,
               basic_conflict,
@@ -1561,6 +1715,9 @@ test_list = [ None,
               basic_auth_cache,
               basic_add_ignores,
               basic_import_ignores,
+              uri_syntax,
+              basic_checkout_file,
+              basic_history,
               ### todo: more tests needed:
               ### test "svn rm http://some_url"
               ### not sure this file is the right place, though.
